@@ -143,8 +143,13 @@ attrs_build (CK_ATTRIBUTE *attrs,
 		}
 
 		memcpy (attr, add, sizeof (CK_ATTRIBUTE));
-		if (!take_values)
-			attr->pValue = memdup (attr->pValue, attr->ulValueLen);
+		if (!take_values && attr->pValue != NULL) {
+			if (attr->ulValueLen == 0)
+				attr->pValue = malloc (1);
+			else
+				attr->pValue = memdup (attr->pValue, attr->ulValueLen);
+			return_val_if_fail (attr->pValue != NULL, NULL);
+		}
 	}
 
 	/* Mark this as the end */
@@ -513,6 +518,11 @@ p11_attr_hash (const void *data)
 	return hash;
 }
 
+static void
+buffer_append_printf (p11_buffer *buffer,
+                      const char *format,
+                      ...) GNUC_PRINTF(2, 3);
+
 static void
 buffer_append_printf (p11_buffer *buffer,
                       const char *format,
@@ -657,6 +667,7 @@ attribute_is_sensitive (const CK_ATTRIBUTE *attr,
 	X (CKA_X_PEER)
 	X (CKA_X_DISTRUSTED)
 	X (CKA_X_CRITICAL)
+	X (CKA_X_PUBLIC_KEY_INFO)
 	X (CKA_NSS_URL)
 	X (CKA_NSS_EMAIL)
 	X (CKA_NSS_SMIME_INFO)
