@@ -64,7 +64,6 @@ struct p11_kit_iter {
 	CK_TOKEN_INFO match_token;
 	CK_ATTRIBUTE *match_attrs;
 	Callback *callbacks;
-	CK_FLAGS session_flags;
 
 	/* The input modules */
 	p11_array *modules;
@@ -85,6 +84,7 @@ struct p11_kit_iter {
 	CK_SLOT_ID slot;
 	CK_SESSION_HANDLE session;
 	CK_OBJECT_HANDLE object;
+	CK_TOKEN_INFO token_info;
 
 	/* And various flags */
 	unsigned int searching : 1;
@@ -93,12 +93,15 @@ struct p11_kit_iter {
 	unsigned int match_nothing : 1;
 	unsigned int keep_session : 1;
 	unsigned int preload_results : 1;
+	unsigned int want_writable : 1;
 };
 
 /**
  * P11KitIterBehavior:
  * @P11_KIT_ITER_BUSY_SESSIONS: Allow the iterator's sessions to be
- * in a busy state when the iterator returns an object.
+ *   in a busy state when the iterator returns an object.
+ * @P11_KIT_ITER_WANT_WRITABLE: Try to open read-write sessions when
+ *   iterating over obojects.
  *
  * Various flags controling the behavior of the iterator.
  */
@@ -124,10 +127,6 @@ p11_kit_iter_new (P11KitUri *uri,
                   P11KitIterBehavior behavior)
 {
 	P11KitIter *iter;
-	CK_ATTRIBUTE *attrs;
-	CK_TOKEN_INFO *tinfo;
-	CK_INFO *minfo;
-	CK_ULONG count;
 
 	iter = calloc (1, sizeof (P11KitIter));
 	return_val_if_fail (iter != NULL, NULL);
@@ -135,6 +134,41 @@ p11_kit_iter_new (P11KitUri *uri,
 	iter->modules = p11_array_new (NULL);
 	return_val_if_fail (iter->modules != NULL, NULL);
 
+	iter->want_writable = !!(behavior & P11_KIT_ITER_WANT_WRITABLE);
+	iter->preload_results = !(behavior & P11_KIT_ITER_BUSY_SESSIONS);
+
+	p11_kit_iter_set_uri (iter, uri);
+	return iter;
+}
+
+/**
+ * p11_kit_iter_set_uri:
+ * @iter: the iterator
+ * @uri: (allow-none): a PKCS\#11 URI to filter on, or %NULL
+ *
+ * Set the PKCS\#11 uri for iterator. Only
+ * objects that match the @uri will be returned by the iterator.
+ * Relevant information in @uri is copied, and you need not keep
+ * @uri around.
+ *
+ * If no @uri is specified then the iterator will iterate over all
+ * objects, unless otherwise filtered.
+ *
+ * This function should be called at most once, and should be
+ * called before iterating begins.
+ *
+ */
+void
+p11_kit_iter_set_uri (P11KitIter *iter,
+                      P11KitUri *uri)
+{
+	CK_ATTRIBUTE *attrs;
+	CK_TOKEN_INFO *tinfo;
+	CK_INFO *minfo;
+	CK_ULONG count;
+
+	return_if_fail (iter != NULL);
+
 	if (uri != NULL) {
 
 		if (p11_kit_uri_any_unrecognized (uri)) {
@@ -154,31 +188,10 @@ p11_kit_iter_new (P11KitUri *uri,
 		}
 	} else {
 		/* Match any module version number*/
+		memset (&iter->match_module, 0, sizeof (iter->match_module));
 		iter->match_module.libraryVersion.major = (CK_BYTE)-1;
 		iter->match_module.libraryVersion.minor = (CK_BYTE)-1;
 	}
-
-	iter->session_flags = CKF_SERIAL_SESSION;
-	iter->preload_results = !(behavior & P11_KIT_ITER_BUSY_SESSIONS);
-
-	return iter;
-}
-
-/**
- * p11_kit_iter_set_session_flags:
- * @iter: the iterator
- * @flags: set of session flags
- *
- * Set the PKCS\#11 session flags to be used when the iterator opens
- * new sessions.
- */
-void
-p11_kit_iter_set_session_flags (P11KitIter *iter,
-                                CK_FLAGS flags)
-{
-	return_if_fail (iter != NULL);
-	return_if_fail (!iter->iterating);
-	iter->session_flags = flags | CKF_SERIAL_SESSION;
 }
 
 /**
@@ -449,7 +462,7 @@ call_all_filters (P11KitIter *iter,
 static CK_RV
 move_next_session (P11KitIter *iter)
 {
-	CK_TOKEN_INFO tinfo;
+	CK_ULONG session_flags;
 	CK_ULONG num_slots;
 	CK_INFO minfo;
 	CK_RV rv;
@@ -493,11 +506,17 @@ move_next_session (P11KitIter *iter)
 		iter->slot = iter->slots[iter->saw_slots++];
 
 		assert (iter->module != NULL);
-		rv = (iter->module->C_GetTokenInfo) (iter->slot, &tinfo);
-		if (rv != CKR_OK || !p11_match_uri_token_info (&iter->match_token, &tinfo))
+		rv = (iter->module->C_GetTokenInfo) (iter->slot, &iter->token_info);
+		if (rv != CKR_OK || !p11_match_uri_token_info (&iter->match_token, &iter->token_info))
 			continue;
 
-		rv = (iter->module->C_OpenSession) (iter->slot, iter->session_flags,
+		session_flags = CKF_SERIAL_SESSION;
+
+		/* Skip if the read/write on a read-only token */
+		if (iter->want_writable && (iter->token_info.flags & CKF_WRITE_PROTECTED) == 0)
+			session_flags |= CKF_RW_SESSION;
+
+		rv = (iter->module->C_OpenSession) (iter->slot, session_flags,
 		                                    NULL, NULL, &iter->session);
 		if (rv != CKR_OK)
 			return finish_iterating (iter, rv);
@@ -655,6 +674,23 @@ p11_kit_iter_get_slot (P11KitIter *iter)
 	return iter->slot;
 }
 
+/**
+ * p11_kit_iter_get_token:
+ * @iter: the iterator
+ *
+ * Get the token info for the token which the current matching object is on.
+ *
+ * This can only be called after p11_kit_iter_next(0 succeeds.
+ *
+ * Returns: the slot of the current matching object.
+ */
+CK_TOKEN_INFO *
+p11_kit_iter_get_token (P11KitIter *iter)
+{
+	return_val_if_fail (iter != NULL, NULL);
+	return &iter->token_info;
+}
+
 /**
  * p11_kit_iter_get_session:
  * @iter: the iterator
@@ -694,6 +730,24 @@ p11_kit_iter_get_object (P11KitIter *iter)
 	return iter->object;
 }
 
+/**
+ * p11_kit_iter_destroy_object:
+ * @iter: teh iterator
+ *
+ * Destory the current matching object.
+ *
+ * This can only be called after p11_kit_iter_next() succeeds.
+ *
+ * Returns: CKR_OK or a failure code
+ */
+CK_RV
+p11_kit_iter_destroy_object (P11KitIter *iter)
+{
+	return_val_if_fail (iter != NULL, CKR_GENERAL_ERROR);
+	return_val_if_fail (iter->iterating, CKR_GENERAL_ERROR);
+	return (iter->module->C_DestroyObject) (iter->session, iter->object);
+}
+
 /**
  * p11_kit_iter_load_attributes:
  * @iter: the iterator
