@@ -518,6 +518,33 @@ test_uri_with_type (void)
 	finalize_and_free_modules (modules);
 }
 
+static void
+test_set_uri (void)
+{
+	CK_FUNCTION_LIST_PTR *modules;
+	P11KitIter *iter;
+	P11KitUri *uri;
+	CK_RV rv;
+
+	modules = initialize_and_get_modules ();
+
+	uri = p11_kit_uri_new ();
+	p11_kit_uri_set_unrecognized (uri, 1);
+	iter = p11_kit_iter_new (NULL, 0);
+	p11_kit_iter_set_uri (iter, uri);
+	p11_kit_uri_free (uri);
+
+	p11_kit_iter_begin (iter, modules);
+
+	/* Nothing should have matched */
+	rv = p11_kit_iter_next (iter);
+	assert_num_eq (rv, CKR_CANCEL);
+
+	p11_kit_iter_free (iter);
+
+	finalize_and_free_modules (modules);
+}
+
 static void
 test_filter (void)
 {
@@ -576,9 +603,7 @@ test_session_flags (void)
 
 	modules = initialize_and_get_modules ();
 
-	iter = p11_kit_iter_new (NULL, 0);
-	p11_kit_iter_set_session_flags (iter, CKF_RW_SESSION);
-
+	iter = p11_kit_iter_new (NULL, P11_KIT_ITER_WANT_WRITABLE);
 	p11_kit_iter_begin (iter, modules);
 
 	while ((rv = p11_kit_iter_next (iter)) == CKR_OK) {
@@ -741,6 +766,37 @@ test_token_mismatch (void)
 	finalize_and_free_modules (modules);
 }
 
+static void
+test_token_info (void)
+{
+	CK_FUNCTION_LIST_PTR *modules;
+	CK_TOKEN_INFO *info;
+	P11KitIter *iter;
+	char *string;
+	CK_RV rv;
+
+	modules = initialize_and_get_modules ();
+
+	iter = p11_kit_iter_new (NULL, 0);
+	p11_kit_iter_begin (iter, modules);
+
+	rv = p11_kit_iter_next (iter);
+	assert_num_eq (rv, CKR_OK);
+
+	info = p11_kit_iter_get_token (iter);
+	assert_ptr_not_null (info);
+
+	string = p11_kit_space_strdup (info->label, sizeof (info->label));
+	assert_ptr_not_null (string);
+
+	assert_str_eq (string, "TEST LABEL");
+
+	free (string);
+	p11_kit_iter_free (iter);
+
+	finalize_and_free_modules (modules);
+}
+
 static void
 test_getslotlist_fail_first (void)
 {
@@ -1126,6 +1182,45 @@ test_many (void *flags)
 	assert (rv == CKR_OK);
 }
 
+static void
+test_destroy_object (void)
+{
+	CK_FUNCTION_LIST **modules;
+	P11KitIter *iter;
+	CK_OBJECT_HANDLE object;
+	CK_SESSION_HANDLE session;
+	CK_FUNCTION_LIST *module;
+	CK_ULONG size;
+	CK_RV rv;
+
+	modules = initialize_and_get_modules ();
+
+	iter = p11_kit_iter_new (NULL, P11_KIT_ITER_WANT_WRITABLE);
+
+	p11_kit_iter_begin (iter, modules);
+
+	/* Should have matched */
+	rv = p11_kit_iter_next (iter);
+	assert_num_eq (rv, CKR_OK);
+
+	object = p11_kit_iter_get_object (iter);
+	session = p11_kit_iter_get_session (iter);
+	module = p11_kit_iter_get_module (iter);
+
+	rv = (module->C_GetObjectSize) (session, object, &size);
+	assert_num_eq (rv, CKR_OK);
+
+	rv = p11_kit_iter_destroy_object (iter);
+	assert_num_eq (rv, CKR_OK);
+
+	rv = (module->C_GetObjectSize) (session, object, &size);
+	assert_num_eq (rv, CKR_OBJECT_HANDLE_INVALID);
+
+	p11_kit_iter_free (iter);
+
+	finalize_and_free_modules (modules);
+}
+
 int
 main (int argc,
       char *argv[])
@@ -1136,6 +1231,7 @@ main (int argc,
 	p11_test (test_all, "/iter/test_all");
 	p11_test (test_unrecognized, "/iter/test_unrecognized");
 	p11_test (test_uri_with_type, "/iter/test_uri_with_type");
+	p11_test (test_set_uri, "/iter/set-uri");
 	p11_test (test_session_flags, "/iter/test_session_flags");
 	p11_test (test_callback, "/iter/test_callback");
 	p11_test (test_callback_fails, "/iter/test_callback_fails");
@@ -1147,6 +1243,7 @@ main (int argc,
 	p11_test (test_keep_session, "/iter/test_keep_session");
 	p11_test (test_token_match, "/iter/test_token_match");
 	p11_test (test_token_mismatch, "/iter/test_token_mismatch");
+	p11_test (test_token_info, "/iter/token-info");
 	p11_test (test_module_match, "/iter/test_module_match");
 	p11_test (test_module_mismatch, "/iter/test_module_mismatch");
 	p11_test (test_getslotlist_fail_first, "/iter/test_getslotlist_fail_first");
@@ -1160,6 +1257,7 @@ main (int argc,
 	p11_test (test_load_attributes_fail_late, "/iter/test_load_attributes_fail_late");
 	p11_testx (test_many, "", "/iter/test-many");
 	p11_testx (test_many, "busy-sessions", "/iter/test-many-busy");
+	p11_test (test_destroy_object, "/iter/destroy-object");
 
 	return p11_test_run (argc, argv);
 }
